module Audio
  ME_POST_FADE_IN = 1000 # milliseconds for bgm to fade back in
  @last_milliseconds = Gosu.milliseconds
  @bgm = nil
  @bgm_fade = -1
  @bgs = nil
  @bgs_fade = -1
  @me  = nil
  @me_fade  = -1
  @me_post_fade_in = -1
  @se  = []
  @song_cache = {}
  @sample_cache = {}
  def self.find_file(filename)
    for path in Settings::SEARCH_PATHS
      unless File.file?(fname = File.join(path, filename))
        possible = Dir.glob(fname + '*')
        if possible.length > 0
          return possible[0] unless File.extname(possible[0]) == '.mid'
        end
      end
    end
    nil
  end
  def self.bgm_play(filename, volume = 100, pitch = 100)
    filename = find_file(filename)
    return bgm_stop if filename.nil?
    unless @song_cache.has_key? filename
      @song_cache[filename] = Gosu::Song.new($GameWindow, filename)
    end
    @bgm = @song_cache[filename]
    @bgm.volume = volume / 100.0
    @bgm.play(true)
  end
  def self.bgm_stop
    @bgm.stop unless @bgm.nil?
  end
  def self.bgm_fade(time)
    @bgm_fade = @bgm.volume / time.to_f unless @bgm.nil?
  end
  def self.bgs_play(filename, volume = 80, pitch = 100)
    filename = find_file(filename)
    return bgs_stop if filename.nil?
    unless @sample_cache.has_key? filename
      @sample_cache[filename] = Gosu::Sample.new($GameWindow, filename)
    end
    @bgs = @sample_cache[filename].play(volume / 100.0, pitch / 100.0, true)
  end
  def self.bgs_stop
    @bgs.stop unless @bgs.nil?
  end
  def self.bgs_fade(time)
    @bgs_fade = @bgs.volume / time.to_f unless @bgs.nil?
  end
  def self.me_play(filename, volume = 100, pitch = 100)
    filename = find_file(filename)
    return me_stop if filename.nil?
    unless @song_cache.has_key? filename
      @song_cache[filename] = Gosu::Song.new($GameWindow, filename)
    end
    unless @bgm.nil?
      @me_post_fade_in = @bgm.volume
      @bgm.volume = 0.0
      @bgm.pause
    end
    @me = @song_cache[filename]
    @me.volume = volume / 100.0
    @me.play(false)
  end
  def self.me_stop
    @me.stop unless @me.nil?
  end
  def self.me_fade(time)
    @me_fade = @me.volume / time.to_f unless @me.nil?
  end
  def self.se_play(filename, volume = 80, pitch = 100)
    filename = find_file(filename)
    unless @sample_cache.has_key? filename
      @sample_cache[filename] = Gosu::Sample.new($GameWindow, filename)
    end
    @se << @sample_cache[filename].play(volume / 100.0, pitch / 100.0, false)
  end
  def self.se_stop
    @se.each { |se| se.stop }
  end
  def self.update
    dt = Gosu.milliseconds - @last_milliseconds
    @last_milliseconds = Gosu.milliseconds
    if !@bgm.nil? and @bgm_fade > -1
      @bgm.volume -= dt * @bgm_fade
      if @bgm.volume <= 0.0
        @bgm.stop
        @bgm_fade = -1
      end
    end
    if !@bgs.nil? and @bgs_fade > -1
      @bgs.volume -= dt * @bgs_fade
      if @bgs.volume <= 0.0
        @bgs.stop
        @bgs_fade = -1
      end
    end
    unless @me.nil?
      if @me_fade > -1
        @me.volume -= dt * @me_fade
        if @me.volume <= 0.0
          @me.stop
          @me_fade = -1
        end
      end
      unless @me.playing?
        if !@bgm.nil? and @bgm.volume < 1.0
          @bgm.play(true) unless @bgm.playing?
          @bgm.volume += @me_post_fade_in / ME_POST_FADE_IN
        else
          @me_post_fade_in = -1
          @me = nil
        end
      end
    end
    for se in @se
      @se.delete(se) unless se.playing?
    end
  end
end
