module Input
  DOWN  = [:KbDown,  :GpDown,  :KbNumpad2]
  LEFT  = [:KbLeft,  :GpLeft,  :KbNumpad4]
  RIGHT = [:KbRight, :GpRight, :KbNumpad6]
  UP    = [:KbUp,    :GpUp,    :KbNumpad8]
  A     = [:KbLeftShift, :KbRightShift, :KbZ, :GpButton0]
  B     = [:KbEscape, :KbNumpad0, :KbX, :GpButton1]
  C     = [:KbSpace, :KbReturn, :KbC, :GpButton2]
  X     = [:KbA, :GpButton3]
  Y     = [:KbS, :GpButton4]
  Z     = [:KbD, :GpButton5]
  L     = [:KbQ, :GpButton6]
  R     = [:KbW, :GpButton7]
  SHIFT = [:KbLeftShift, :KbRightShift]
  CTRL  = [:KbLeftCtrl,  :KbRightCtrl]
  ALT   = [:KbLeftAlt,   :KbRightAlt]
  F1    = [:KbF1]
  F2    = [:KbF2]
  F3    = [:KbF3]
  F4    = [:KbF4]
  F5    = [:KbF5]
  F6    = [:KbF6]
  F7    = [:KbF7]
  F8    = [:KbF8]
  F9    = [:KbF9]
  F10   = [:KbF10]
  F11   = [:KbF11]
  F12   = [:KbF12]
  @press    = Hash.new(0)
  @disabled = Hash.new(0)
  @force    = Hash.new(0)
  def self.update
    Gosu::Button.constants.sort.each do |const|
      sym = const.to_sym
      next unless (button = Gosu::Button.const_get(sym)).is_a? Integer
      test = $GameWindow.button_down?(button)
      test |= (@force[sym].nil? or @force[sym] > 0)
      test &= !(@disabled[sym].nil? or @disabled[sym] > 0)
      @press[sym] = (test ? @press[sym] + 1 : 0)
      @disabled[sym] -= 1 if !@disabled[sym].nil? and @disabled[sym] > 0
      @force[sym] -= 1 if !@force[sym].nil? and @force[sym] > 0
    end
    if press?(ALT) and @press[:KbReturn] == 1
      Graphics.fullscreen = !Graphics.fullscreen
    end
    if @press[:KbF2] == 1
      $GameWindow.showing_fps = !$GameWindow.showing_fps
    end
  end
  def self.press?(key)
    key.each { |sym| return true if @press[sym] > 0 }
    false
  end
  def self.trigger?(key)
    key.each { |sym| return true if @press[sym] == 1 }
    false
  end
  def self.repeat?(key)
    key.each do |sym|
      return true if @press[sym] == 1
      return true if @press[sym] > 18 and @press[sym] % 6 == 0
    end
    false
  end
  def self.dir4
    return 8 if press?(UP)
    return 6 if press?(RIGHT)
    return 4 if press?(LEFT)
    return 2 if press?(DOWN)
    0
  end
  def self.dir8
    return 9 if @press[:KbNumpad9] or (press?(UP) and press?(RIGHT))
    return 7 if @press[:KbNumpad7] or (press?(UP) and press?(LEFT))
    return 3 if @press[:KbNumpad3] or (press?(DOWN) and press?(RIGHT))
    return 1 if @press[:KbNumpad1] or (press?(DOWN) and press?(LEFT))
    self.dir4
  end
  def self.disable_key(key, frames = nil)
    key.each { |sym| @disabled[sym] = frames }
  end
  def self.enable_key(key)
    key.each { |sym| @disabled[sym] = 0 }
  end
  def self.press_key(key, frames = 1)
    key.each { |sym| @force[sym] = frames }
  end
end
