# Rewritten the Table class. This is slower than RGSS but more portable.

class Table
  # Initialize the table with variable length arguments
  def initialize(xsize, ysize = 1, zsize = 1)
    resize(xsize, ysize, zsize)
  end

  # Set the size of (and number of) the dimensions; clears the table
  def resize(xsize, ysize = 1, zsize = 1)
    @data = Array.new(xsize * ysize * zsize, 0)
    @xsize, @ysize, @zsize = xsize, ysize, zsize
  end

  # Returns the size at a specific depth, or 1 if out of range
  def xsize ; @xsize ; end
  def ysize ; @ysize ; end
  def zsize ; @zsize ; end

  # Retrieve data
  def [](x, y = 0, z = 0)
    # Return nil if out of range
    return nil if (i = get_index(x, y, z)).nil?
    @data[i]
  end

  # Set data
  def []=(*args)
    # Remove the last argument, which is what the data will be set to
    operand = args.pop.to_i
    # Return nil if out of range
    return nil if (i = get_index(*args).nil?)
    @data[i] = operand
  end

  # Check arguments for size and within range
  def get_index(x, y, z)
    return nil if x < 0 or x >= xsize
    return nil if y < 0 or y >= ysize
    return nil if z < 0 or z >= zsize
    (z * @ysize * @xsize) + (y * @xsize) + x
  end

  # Dump to String (compatible with RGSS Table dump format for Integers)
  def _dump(marshal_depth = -1)
    # First, store information about the size
    size_info = [@sizes.length, *@sizes]
    size_info << 0x01 while size_info.length < 4
    output = size_info.pack('I*')
    # Then, store some empty data
    output += [0xD2].pack('I')
    # All values are integers (shorts), so store them as such
    output += @data.pack('s*')
    output
  end

  # Load from String (compatible with RGSS Table dump format for Integers)
  def self._load(data)
    # Get depth of the Table
    depth = data.slice!(0...4).unpack('I')[0]
    # Get size of each dimension
    sizes = data.slice!(0...[12, depth * 4].max).unpack('I*')[0...depth]
    output = Table.new(*sizes)
    data.slice!(0...4) # Skip this weird data
    # Integer format: load each piece of data in the correct order
    output.instance_variable_set(:@data, data.unpack('s*'))
    return output
  rescue RangeError
    # Catches failed attempts to slice existing data due to out of range
    raise RuntimeError, 'Load data for Table is not as long as specified. ' +
                        'The file may be corrupted.'
  end

  # Process through each element in order
  def each(&block)
    @data.each do |val|
      args = [val]
      args << i / (@ysize*@zsize) if block.arity > 1
      args << (i % @zsize) / @ysize if block.arity > 2
      args << i % (@ysize*@zsize) if block.arity > 3
      yield *args
    end
  end
end
 